@@ -23,18 +23,18 @@ void main()
     for (l = 0; l < 7; l++)
       lambda[l] = 0.0;
   
-    title("(DV, P) vs (D, V, P)\nParameters: mu D V P DV DP VP");
-    categ("cda171", "n def vic pen", &loglin, lambda, "1111?00", "", "");
+    title("(DV, P) vs (D, V, P)");
+    categ("cda171", "n def vic pen", &loglin, lambda, "mu D V P ?DV !DP !VP", "", "");
     sort("cda171.cat", "def vic _type_ pen", "");
     table("cda171.cat.srt", "def vic", "_type_ pen n", "s6.2 30", "");
   
-    title("(DV, VP) vs (DV, P)\nParameters: mu D V P DV DP VP");
-    categ("cda171", "n def vic pen", &loglin, lambda, "111110?", "", "");
+    title("(DV, VP) vs (DV, P)");
+    categ("cda171", "n def vic pen", &loglin, lambda, "mu D V P DV !DP ?VP", "", "");
     sort("cda171.cat", "def vic _type_ pen", "");
     table("cda171.cat.srt", "def vic", "_type_ pen n", "s6.2 30", "");
   
-    title("(DV, DP, VP) vs (DV, VP)\nParameters: mu D V P DV DP VP");
-    categ("cda171", "n def vic pen", &loglin, lambda, "11111?1", "", "");
+    title("(DV, DP, VP) vs (DV, VP)");
+    categ("cda171", "n def vic pen", &loglin, lambda, "mu D V P DV ?DP VP", "", "");
     sort("cda171.cat", "def vic _type_ pen", "");
     table("cda171.cat.srt", "def vic", "_type_ pen n", "s6.2 30", "");
   }
