@@ -2,7 +2,7 @@
  * compiling, and running Dap programs.  
  */
 
-/*  Copyright (C) 2001 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
  *
  *  Dap is a GNU program
  *
@@ -85,7 +85,7 @@ int keep;	/* flag: keep .lst file and append to it each run? */
 int debug;	/* flag: save .dap.c for debugging? */
 int runstat;	/* return status of execution of program */
 
-fputs("\nDap, Copyright (C) 2001 Free Software Foundation, Inc.\n", stderr);
+fputs("\nDap, Copyright (C) 2001, 2002 Free Software Foundation, Inc.\n", stderr);
 fputs("Dap comes with ABSOLUTELY NO WARRANTY;\n", stderr);
 fputs("for details see the GNU Public License.\n", stderr);
 fputs("This is free software, and you are welcome to\n", stderr);
@@ -111,12 +111,12 @@ while (argv[1][0] == '-')
 		{
 		fputs("Usage:\ndap [-k] [-d] FILE1.c [ FILE2.c ... ] [-a ARG1 ...]\n", stderr);
 		fputs("dap [--keep] [--debug] FILE1.c [ FILE2.c ... ] [--args] ARG1 ...]\n", stderr);
-		fputs("\nReport bugs to <sbassein@earthlink.net>\n", stderr);
+		fputs("\nReport bugs to <bug-dap@gnu.org>\n", stderr);
 		exit(1);
 		}
 	else if (!strcmp(argv[0], "--version"))
 		{
-		fputs("Dap 1.1\n", stderr);
+		fputs("Dap 1.6\n", stderr);
 		exit(1);
 		}
 	else
