@@ -1,6 +1,6 @@
 /* Contains the basic I/O and setup routines.  */
 
-/*  Copyright (C) 2001 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
  *
  *  Dap is a GNU program.
  *
@@ -511,7 +511,7 @@ for (v = 0; v < dap_obs[dap_ono].do_nvar; v++)
 			for (n = 0; vname[n] &&
 					vname[n] == dap_obs[dap_ono].do_nam[v + d][n]; n++)
 				;
-			if (vname[n] || dap_obs[dap_ono].do_nam[v][n] != '[')
+			if (vname[n] || dap_obs[dap_ono].do_nam[v + d][n] != '[')
 				break;
 			}
 		*dim = d;
@@ -1247,7 +1247,7 @@ if (!strcmp(action, "RENAME"))
 	}
 else if (!strncmp(action, "COPY", 4))
 	{
-	for (v = 4; action[v] == ' '; v++)
+	for (v = 4; action[v] == ' '; v++)	/* get past COPY */
 		;
 	for (nvar = 0; action[v]; nvar++)
 		{
@@ -1262,10 +1262,10 @@ else if (!strncmp(action, "COPY", 4))
 				exit(1);
 				}
 			}
-		oldvar[nvar][c] = '\0';
+		oldvar[nvar][c] = '\0';		/* now oldvar[nvar] is named variable */
 		while (action[v] == ' ')
 			v++;
-		if (action[v] == '>')
+		if (action[v] == '>')		/* if changing name */
 			{
 			for (v++; action[v] == ' '; v++)
 				;
@@ -1281,51 +1281,64 @@ else if (!strncmp(action, "COPY", 4))
 					exit(1);
 					}
 				}
-			newvar[nvar][c] = '\0';
+			newvar[nvar][c] = '\0';	/* now newvar[nvar] is new name */
 			while (action[v] == ' ')
 				v++;
 			}
 		else
 			strcpy(newvar[nvar], oldvar[nvar]);
 		}
-	if (nvar)
+	if (nvar)		/* if there are variables to copy */
 		{
 		inset(oldname);
 		for (v = 0, outlist[0] = '\0'; v < nvar; v++)
 			{
 			if (v)
 				strcat(outlist, " ");
-			strcat(outlist, newvar[v]);
-			if ((vn = dap_varnum(oldvar[v])) >= 0)
+			strcat(outlist, newvar[v]);	/* output list has newvars */
+			if (strcmp(oldvar[v], newvar[v]))	/* if changing name */
 				{
-				if ((nv = dap_varnum(newvar[v])) >= 0)
-					dap_obs[0].do_nam[nv][0] = '_';
-				strcpy(dap_obs[0].do_nam[vn], newvar[v]);
-				}
-			else if ((vn = dap_arrnum(oldvar[v], &dim)) >= 0)
-				{
-				if ((nv = dap_arrnum(newvar[v], &ndim)) >= 0)
+				if ((vn = dap_varnum(oldvar[v])) >= 0)
 					{
-					for (c = 0; c < ndim; c++)
+				/* if newvar exists in oldname, zap its first char to '0' */
+					if ((nv = dap_varnum(newvar[v])) >= 0)
+						dap_obs[0].do_nam[nv][0] = '0';
+				/* and change the name of oldvar to newvar */
+					dap_free(dap_obs[0].do_nam[vn]);
+					dap_obs[0].do_nam[vn] =
+						dap_malloc(strlen(newvar[v]) + 1, "");
+					strcpy(dap_obs[0].do_nam[vn], newvar[v]);
+					}
+				else if ((vn = dap_arrnum(oldvar[v], &dim)) >= 0)
+					{
+				/* do the same for arrays */
+					if ((nv = dap_arrnum(newvar[v], &ndim)) >= 0)
+						{
+						for (c = 0; c < ndim; c++)
+							{
+							dap_obs[0].do_nam[nv + c][0] = '0';
+							sprintf(dimstr, "[%d]", c);
+							strcat(dap_obs[0].do_nam[vn + c], dimstr);
+							}
+						}
+					for (c = 0; c < dim; c++)
 						{
-						dap_obs[0].do_nam[nv + c][0] = '_';
 						sprintf(dimstr, "[%d]", c);
+						dap_free(dap_obs[0].do_nam[vn + c]);
+						dap_obs[0].do_nam[vn + c] =
+							dap_malloc(strlen(newvar[v]) +
+									strlen(dimstr) + 1, "");
+						strcpy(dap_obs[0].do_nam[vn + c], newvar[v]);
 						strcat(dap_obs[0].do_nam[vn + c], dimstr);
 						}
 					}
-				for (c = 0; c < dim; c++)
+				else
 					{
-					strcpy(dap_obs[0].do_nam[vn + c], newvar[v]);
-					sprintf(dimstr, "[%d]", c);
-					strcat(dap_obs[0].do_nam[vn + c], dimstr);
+					fprintf(dap_err, "(dataset) unknown variable %s\n",
+								oldvar[v]);
+					exit(1);
 					}
 				}
-			else
-				{
-				fprintf(dap_err, "(dataset) unknown variable %s\n",
-							oldvar[v]);
-				exit(1);
-				}
 			}
 		outset(newname, outlist);
 		while (step())
@@ -1740,6 +1753,7 @@ int inbrack;
 static char *vname;
 int dim;
 static int *outv;
+int l0;
 int l1;
 int f1;
 int nv;
@@ -1778,7 +1792,14 @@ for (inbrack = 0, l = 0, f = 0; varl[l]; l++)
 varlist[f] = '\0';
 for (l = 0; varlist[l] == ' '; l++)
 	;
-for (nv = 0, l1 = 0; varlist[l]; )
+l0 = 0;
+if (varlist[l] == '!')
+	{
+	for (l++; varlist[l] == ' '; l++)
+		;
+	varlist[l0++] = '!';
+	}
+for (nv = 0, l1 = l0; varlist[l]; )
 	{
 	for (f = 0; varlist[l + f] && varlist[l + f] != ' '; f++)
 		{
@@ -1805,7 +1826,7 @@ for (nv = 0, l1 = 0; varlist[l]; )
 	if (vn == nv)				/* if not */
 		{
 		outv[nv++] = v;			/* enter it in list of indices */
-		if (l1 > 0)
+		if (l1 > l0)
 			varlist[l1++] = ' ';
 		for (f1 = 0; f1 < f; f1++)	/* copy name from varlist to varlist */
 			varlist[l1++] = varlist[l++];
@@ -1826,6 +1847,7 @@ int i;
 static char *varlist;
 static char *vname;
 int v;
+int w;
 int first;
 int dim;
 int d;
@@ -1862,27 +1884,90 @@ if (varlist[0])
 	{
 	for (l = 0; varlist[l] == ' '; l++)
 		;
-	while (varlist[l])
+	if (varlist[l] == '!')	/* listing variables NOT to include */
+		{		/* first put them all in */
+		for (l++; varlist[l] == ' '; l++)
+			;
+		dap_obs[dap_ono].do_ovar = dap_obs[dap_ono].do_nvar;
+		for (v = 0; v < dap_obs[dap_ono].do_nvar; v++)
+			dap_obs[dap_ono].do_out[v] = v;
+		while (varlist[l])	/* then take them out */
+			{
+			for (i = 0; varlist[l + i] && varlist[l + i] != ' '; i++)
+				vname[i] = varlist[l + i];
+			vname[i] = '\0';
+			if ((w = dap_varnum(vname)) >= 0)
+				{
+				for (v = 0; v < dap_obs[dap_ono].do_ovar; v++)
+					{
+					if (dap_obs[dap_ono].do_out[v] == w)
+						break;
+					}
+				if (v == dap_obs[dap_ono].do_ovar)
+					{
+					fprintf(dap_err,
+			"(outset) variable not in list of variables to exclude: %s\n",
+						vname);
+					exit(1);
+					}
+				while (v < dap_obs[dap_ono].do_ovar - 1)
+					{
+					dap_obs[dap_ono].do_out[v] =
+						dap_obs[dap_ono].do_out[v + 1];
+					v++;
+					}
+				dap_obs[dap_ono].do_ovar--;
+				}
+			else if ((w = dap_arrnum(vname, &dim)) >= 0)
+				{
+				for (v = 0; v < dap_obs[dap_ono].do_ovar; v++)
+					{
+					if (dap_obs[dap_ono].do_out[v] == w)
+						break;
+					}
+				while (v < dap_obs[dap_ono].do_ovar - dim)
+					{
+					dap_obs[dap_ono].do_out[v] =
+						dap_obs[dap_ono].do_out[v + dim];
+					v++;
+					}
+				dap_obs[dap_ono].do_ovar -= dim;
+				}
+			else
+				{
+				fprintf(dap_err, "(outset(%s)) unknown variable: %s\n",
+					fname, vname);
+				exit(1);
+				}
+			l += i;
+			while (varlist[l] == ' ')
+				l++;
+			}
+		}
+	else
 		{
-		for (i = 0; varlist[l + i] && varlist[l + i] != ' '; i++)
-			vname[i] = varlist[l + i];
-		vname[i] = '\0';
-		if ((v = dap_varnum(vname)) >= 0)
-			dap_obs[dap_ono].do_out[dap_obs[dap_ono].do_ovar++] = v;
-		else if ((v = dap_arrnum(vname, &dim)) >= 0)
+		while (varlist[l])
 			{
-			for (d = 0; d < dim; d++)
-				dap_obs[dap_ono].do_out[dap_obs[dap_ono].do_ovar++] = v + d;
+			for (i = 0; varlist[l + i] && varlist[l + i] != ' '; i++)
+				vname[i] = varlist[l + i];
+			vname[i] = '\0';
+			if ((v = dap_varnum(vname)) >= 0)
+				dap_obs[dap_ono].do_out[dap_obs[dap_ono].do_ovar++] = v;
+			else if ((v = dap_arrnum(vname, &dim)) >= 0)
+				{
+				for (d = 0; d < dim; d++)
+					dap_obs[dap_ono].do_out[dap_obs[dap_ono].do_ovar++] = v + d;
+				}
+			else
+				{
+				fprintf(dap_err, "(outset(%s)) unknown variable: %s\n",
+					fname, vname);
+				exit(1);
+				}
+			l += i;
+			while (varlist[l] == ' ')
+				l++;
 			}
-		else
-			{
-			fprintf(dap_err, "(outset(%s)) unknown variable: %s\n",
-				fname, vname);
-			exit(1);
-			}
-		l += i;
-		while (varlist[l] == ' ')
-			l++;
 		}
 	}
 else
@@ -1891,6 +1976,24 @@ else
 	for (v = 0; v < dap_obs[dap_ono].do_nvar; v++)
 		dap_obs[dap_ono].do_out[v] = v;
 	}
+if (dap_varnum("_type_") < 0)
+	{
+	fputs("(outset) missing _type_ variable\n", dap_err);
+	exit(1);
+	}
+for (v = 0; v < dap_obs[dap_ono].do_ovar; v++)
+	{
+	for (w = v + 1; w < dap_obs[dap_ono].do_ovar; w++)
+		{
+		if (dap_obs[dap_ono].do_out[v] == dap_obs[dap_ono].do_out[w])
+			{
+			fprintf(dap_err,
+			"(outset) duplicate variable in output list: %s\n",
+				dap_obs[dap_ono].do_nam[dap_obs[dap_ono].do_out[w]]);
+			exit(1);
+			}
+		}
+	}
 for (v = 0, first = 1; v < dap_obs[dap_ono].do_ovar; v++)
 	{
 	if (!first)
@@ -1911,7 +2014,7 @@ int first;
 
 if ((v = dap_varnum("_type_")) < 0)
 	{
-	fprintf(dap_err, "(output) missing _type_\n");
+	fprintf(dap_err, "(output) missing _type_ variable\n");
 	exit(1);
 	}
 for (v = 0, first = 1; v < dap_obs[dap_ono].do_ovar; v++)
