@@ -1,6 +1,6 @@
 /* dap1.c -- sort and table */
 
-/*  Copyright (C) 2001 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -109,7 +109,7 @@ int i;
 int v;
 int vn;
 static char *vname;
-char sortarg[12];
+char sortarg[15];
 char *dsrt0;
 char *dsrt;
 char *dfile;
@@ -232,7 +232,7 @@ else
 		vname[i] = '\0';
 		if ((v = dap_varnum(vname)) >= 0)
 			{
-			sprintf(sortarg, " +%d", v);
+			sprintf(sortarg, " -k %d,%d", v + 1, v + 1);
 			strcat(command, sortarg);
 			if (nmods)
 				{
@@ -247,8 +247,6 @@ else
 					exit(1);
 					}
 				}
-			sprintf(sortarg, " -%d", v + 1);
-			strcat(command, sortarg);
 			}
 		else
 			{
@@ -1201,7 +1199,7 @@ for (c = 0; label[c] && c < width; c++)
 while (c < width)
 	label[c++] = ' ';
 label[c] = '\0';
-fprintf(dap_lst, label);
+fprintf(dap_lst, "%s", label);
 }
 
 static void divider(int left, int conn, int sep, int right, int nblank)
