@@ -1,6 +1,6 @@
 /* ps1.c -- higher level functions */
 
-/*  Copyright (C) 2001 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,11 +35,21 @@ extern FILE *dap_err;
 
 extern char *pict_newstr(char *str);
 
-static double arint(double x)
+static double arintup(double x)
 {
 double i;
 
-if (fabs(i = rint(x)) == 0.0)
+if (fabs(i = ceil(x)) == 0.0)
+	return 0.0;
+else
+	return i;
+}
+
+static double arintdown(double x)
+{
+double i;
+
+if (fabs(i = floor(x)) == 0.0)
 	return 0.0;
 else
 	return i;
@@ -293,10 +303,10 @@ double pict_autoaxes(pict *p, char *xlab, char *ylab, char *axspec,
 pict *pp;
 int totpts;
 double xspace, yspace;
-double xlen, ylen;
-double minx, maxx, miny, maxy;
+static double xlen, ylen;
+static double minx, maxx, miny, maxy;
 double minx1, maxx1;
-int nxticks, nyticks;
+static int nxticks, nyticks;
 tick *xticks;
 tick *yticks;
 double xticklen;
@@ -490,20 +500,24 @@ if (maxy < 0.0 && !finite(specymax))
 	maxy = 0.0;
 xlen = 1e5 / (maxx - minx);
 if (!finite(specxmin))
-	minx = arint(xlen * minx) / xlen;
+	minx = arintdown(xlen * minx) / xlen;
 if (!finite(specxmax))
-	maxx = arint(xlen * maxx) / xlen;
+	maxx = arintup(xlen * maxx) / xlen;
 ylen = 1e5 / (maxy - miny);
 if (!finite(specymin))
-	miny = arint(ylen * miny) / ylen;
+	miny = arintdown(ylen * miny) / ylen;
 if (!finite(specymax))
-	maxy = arint(ylen * maxy) / ylen;
+	maxy = arintup(ylen * maxy) / ylen;
 xlen = maxx - minx;
+if (specxticks > 2)
+	nxticks = specxticks - 2;
+else
+	nxticks = 10.0;
 if (xlen >= 1.0)
         {
-        for (xspace = 1.0; 10.0 * xspace < xlen; xspace *= 10.0)
+        for (xspace = 1.0; nxticks * xspace < xlen; xspace *= nxticks)
                 ; 
-        xspace *= ceil(xlen / xspace) / 10.0;
+        xspace *= ceil(xlen / xspace) / nxticks;
         }
 else            
         {
@@ -512,11 +526,15 @@ else
         xspace *= ceil(xlen / xspace) / 10.0;
         }
 ylen = maxy - miny;
+if (specyticks > 0)
+	nyticks = specyticks;
+else
+	nyticks = 10.0;
 if (ylen >= 1.0)
         {
-        for (yspace = 1.0; 10.0 * yspace < ylen; yspace *= 10.0)
+        for (yspace = 1.0; nyticks * yspace < ylen; yspace *= nyticks)
                 ;
-        yspace *= ceil(ylen / yspace) / 10.0;
+        yspace *= ceil(ylen / yspace) / nyticks;
         }
 else
         {
@@ -547,8 +565,7 @@ if (maxx > maxx1)
 else
 	maxx = xspace * (ceil(maxx / xspace) + 1.0);
 rexpand = (maxx - xspace >= maxx1);
-if (!finite(specymax))
-	maxy = yspace * ceil(maxy / yspace);
+maxy = yspace * ceil(maxy / yspace);
 xticklen = (maxy - miny) / 100.0;
 yticklen = (maxx - minx) / 100.0;
 if (specxticks > 0)
